<?php

/**
 * Definición de la clase Buffer
 * @package Core
 * @version 3.0
 * @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License
 * @copyright TuApp.net 2011
 */

/**
 * Maneja un arreglo de buffers de salida
 *
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright TuApp.net 2011
 * @package Core
 * @version 3.0
 */

class Buffer
{	
	/**
	 * Instancia única de la clase
	 * @var Buffer
	 * @static
	 */
	private static $instance;
	
	/**
	 * Arreglo que almacena los buffers de salida
	 * @var array
	 */
	private $buffer = array();

	/**
	 * Pila que almacena los identificadores de los buffers
	 * @var array
	 */
	private $stack = array();

	/**
	 * Constructor privado para evitar otras instancias
	 * @return void
	 */
	private function __construct()
	{
	}
	
	/**
     * Invalida la clonación de la clase
     * @return void
     * @final
     */
   	final public function __clone()
   	{
     	trigger_error("Operación Invalida: No puedes clonar una instancia de ". get_class($this) .".", E_USER_ERROR );
   	}
  
  	/**
  	 * Invalida la deserialización de la clase
  	 * @return void
  	 * @final
  	 */
   	final public function __wakeup()
   	{
      	trigger_error("No puedes deserializar una instancia de ". get_class($this) .".", E_USER_ERROR );
   	}

	/**
	 * Activa un buffer de salida con el indentificador asignado
	 * @param string $buffer_id
	 * @return void
	 */
	public function start( $buffer_id )
	{
		$this->stack[] = $buffer_id;

		ob_start();
	}

	/**
	 * Finaliza el buffering de la salida y almacena su contenido
	 * @return void
	 */
	public function end()
	{
		$buffer_id = array_pop( $this->stack );

		$contents = ob_get_contents();

		if( $contents && $buffer_id )
		{
			if( isset( $this->buffer[ $buffer_id ] ))
			{
				$this->buffer[ $buffer_id ] .= $contents;
			}
			else
			{
				$this->buffer[ $buffer_id ] = $contents;
			}
			
			ob_end_clean();
		}
	}
	
	/**
	 * Devuelve el contenido del buffer de salida que se encuentra almacenado con el id especificado
	 * @param string $buffer_id
	 * @param boolean[optional] $clean Limpia el contenido del buffer, por defecto es verdadera
	 * @return string Si el buffer no existe devuelve una cadena vacía
	 */
	public function getContent( $buffer_id, $clean = true )
	{
		if( !isset( $this->buffer[$buffer_id] ) )
		{
			return '';
		}
		
		$buffer = $this->buffer[$buffer_id];
		
		if( $clean )
		{
			$this->buffer[$buffer_id] = '';
		}
		
		return $buffer;		
	}
	
	/**
	 * Limpia el contenido almacenado del buffer con el id especificado
	 * @param string $buffer_id
	 * @return void
	 */
	public function clean( $buffer_id )
	{
		$this->buffer[$buffer_id] = '';
	}
	
	/**
	 * Escribe el contenido directamente en el buffer almacenado
	 * @param string $buffer_id
	 * @param string $content
	 * @param boolean[optional] $overwrite Si es verdadero sobreescribe el contenido anterior. Por defecto es falso.
	 * @return void
	 */
	public function write( $buffer_id, $content, $overwrite = false )
	{
		if( !isset($this->buffer[$buffer_id]) || $overwrite )
		{
			$this->buffer[$buffer_id] = $content;
		}
		else
		{
			$this->buffer[$buffer_id] .= $content;
		}
	}
		
	/**
	 * Devuelve la instancia única de la clase
	 * @return Buffer
	 * @static
	 */
	public static function getInstance()
	{
		if( !self::$instance instanceof self )
		{
			self::$instance = new self;
		}

		return self::$instance;
	}
}

// Fin de archivo
