<?php

/**
 * Definición de la clase Controller
 * @package Core
 * @version 3.0
 * @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License
 * @copyright TuApp.net 2011
 */

/**
 * Controlador base para MoonDragon
 *
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright TuApp.net 2011
 * @package Core
 * @version 3.0
 * @abstract
 */

abstract class Controller
{
	/**
	 * Tarea por defecto
	 * @var string
	 */
	protected $default_task;

	/**
	 * Vista utilizada por el controlador
	 * @var View
	 */
	protected $view;

	/**
	 * Constructor base del controlador
	 * @param string[optional] $default_task
	 * @return void
	 */
	public function __construct( $default_task = '' )
	{
		$this->default_task = ( $default_task == '' )?'display':$default_task;
		if(!($this->view instanceof View))
		{
			$view = Registry::get('DEFAULT_VIEW').'View';
			$this->view = new $view();
		}
				
		Registry::set( 'controller', $this );
	}

	/**
	 * Procesa las llamadas a las tareas desconocidas
	 * @param string $name
	 * @param array $params
	 * @return void
	 */
	public function __call( $name, $params )
	{
		trigger_error( 'La tarea '.$name.' no existe para el controlador '.get_class($this), E_USER_WARNING );
	}

	/**
	 * Modifica la tarea por defecto del controlador
	 * @param string $task
	 * @return void
	 */
	public function setDefaultTask( $task )
	{
		$this->default_task = ($task == '' )?'display':$task;
	}

	/**
	 * Registra un modelo para que sea usado en el controlador
	 * @param string $model
	 * @param string[optional] $name
	 * @return void
	 */
	public function registerModel( $model, $name = '' )
	{
		$class = $model.'Model';
		
		if( !( class_exists( $class ) && is_subclass_of( $class, 'Model' ) ) )
		{
			if( file_exists( MODELS.'/class.'.strtolower($class).'.php' ) )
			{
				include_once MODELS.'/class.'.strtolower($class).'.php';
			}
			
			if( !( class_exists( $class ) && is_subclass_of( $class, 'Model' ) ) )
			{
				trigger_error( 'No existe el modelo '.$model , E_USER_NOTICE );
			}
		}
		
		if($name == '')
		{
			$name = strtolower($model);
		}
		
		if( !($this->$name instanceof $class) )
		{
			$this->$name = new $class();
		}
	}
	
	/**
	 * Registra la vista utilizada por el controlador
	 * @param string $view
	 * @return void
	 */
	public function registerView( $view )
	{
		$view .= 'View';
		
		if( !( class_exists( $view ) && is_subclass_of( $view, 'View' ) ) )
		{
			if( file_exists( VIEWS.'/class.'.strtolower($view).'.php' ) )
			{
				include_once VIEWS.'/class.'.strtolower($view).'.php';
			}
			
			if( !( class_exists( $view ) && is_subclass_of( $view, 'View' ) ) )
			{
				trigger_error( 'No existe la vista '.$view , E_USER_NOTICE );
				$view = Registry::get( 'DEFAULT_VIEW' ).'View';
			}
		}
	
		if( !($this->view instanceof $view) )
		{
			$this->view = new $view();
		}
	}
	
	/**
	 * Devuelve la vista utilizada por el controlador
	 * @return View
	 */
	public function getView()
	{
		if(!$this->view instanceof View)
		{
			$view = Registry::get('DEFAULT_VIEW').'View';
			$this->view = new $view();
		}
		return $this->view;
	}

	/**
	 * Ejecuta la tarea por defecto o la tarea seleccionada
	 * @param string[optional] $task
	 * @return void
	 */
	public function run( $task = '' )
	{
		if( !$this->authenticate() )
		{
			$task = 'unauthorized';
		}
		
		if( Registry::get( 'OFFLINE_ERROR' ) && !Registry::get( 'online' ) )
		{
			$task = 'offline';
		}
		
		if( Registry::get( 'unexistent') )
		{
			$task = 'unexistent';
		}
		
		$mdview = $this->getView();
		$_buffer = Buffer::getInstance();
		$_buffer->start( 'salida' );
		
		if( $task == '' )
		{
			$default_task = $this->default_task;
			$this->$default_task();
		}
		else
		{
			$this->$task();
		}
		
		$_buffer->end();
	}

	/**
	 * Tarea por defecto
	 * @return void
	 * @abstract
	 */
	abstract protected function display();
	
	/**
	 * Procesamiento de secciones no autorizadas
	 * @return void
	 */
	protected function unauthorized()
	{
		$view = $this->getView();
		$view->message( 'No esta autorizado a ver esta sección' ); // @todo localizar contenido
	}
	
	/**
	 * Procesamiento de secciones no existentes
	 * @return void
	 */
	protected function unexistent()
	{
		$view = $this->getView();
		$view->message( 'La sección seleccionada no existe' ); // @todo localizar contenido
	}
	
	/**
	 * Error al no existir conexión con la base de datos
	 * @return void
	 */
	protected function offline()
	{
		$view = $this->getView();
		$view->message( 'Imposible conectarse con la base de datos' ); // @todo localizar contenido		
	}

	/**
	 * Comrpueba si el sistema esta autorizado para continuar
	 * @return boolean
	 */
	protected function authenticate()
	{
		return true;
	}
}

//Fin de archivo
