<?php

/**
 * Definición de la clase Database
 * @package Core
 * @version 3.0
 * @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License
 * @copyright TuApp.net 2011
 */

/**
 * Clase para manejar la conexion de base de datos
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright TuApp.net 2011
 * @package Core
 * @version 3.0
 */

class Database
{
	/**
	 * Los resultados se muestran en un objeto
	 */
	const F_OBJECT = 101;
	
	/**
	 * Los resultados se muestran en una matriz numérica
	 */
	const F_ROW = 102;

	/**
	 * Los resultados se muestran en una matriz asociativa
	 */
	const F_ASSOC = 103;

	/**
	 * Instancia única de la clase
	 * @var Database
	 * @static
	 */
	private static $instance;
	
	/**
	 * Perfiles de conexión
	 * @var array
	 */
	private $profiles = array();
	
	/**
	 * Perfil de conexión activo
	 * @var string
	 */
	private $selected = 'default';
	
	/**
	 * Bandera para controlar el estado de la conexion
	 * @var boolean
	 */
	private $connected = false;
	
	/**
	 * Guarda las consultas que se han hecho a la base de datos
	 * @var string
	 */
	private $query_history = '';
	
	/**
	 * Mantiene el recurso de conexion
	 * @var resource
	 */
	private $connection;

	/**
	 * Constructor privado para evitar otras instancias
	 * @return void
	 */
	private function __construct()
	{
		$this->profiles[$this->selected]['host'] = 'localhost';
		$this->profiles[$this->selected]['user'] = 'root';
		$this->profiles[$this->selected]['password'] = '';
		$this->profiles[$this->selected]['database'] = 'test';
   	}

   	/**
     * Invalida la clonación de la clase
     * @return void
     * @final
     */
   	final public function __clone()
   	{
     	trigger_error("Operación Invalida: No puedes clonar una instancia de ". get_class($this) .".", E_USER_ERROR );
   	}
  
  	/**
  	 * Invalida la deserialización de la clase
  	 * @return void
  	 * @final
  	 */
   	final public function __wakeup()
   	{
      	trigger_error("No puedes deserializar una instancia de ". get_class($this) .".", E_USER_ERROR );
   	}
	
	/**
	 * Asigna el dominio de la base de datos
	 * @param string $host
	 * @param string[optional] $profile
	 * @return void
	 */
	private function setHost( $host, $profile = '' )
	{
		$profile = ($profile == '' )?$this->selected:$profile;
		$this->profiles[$profile]['host'] = $host;
	}

	/**
	 * Asigna el usuario de la base de datos
	 * @param string $user
	 * @param string[optional] $profile
	 * @return void
	 */
	private function setUser( $user, $profile = '' )
	{
		$profile = ($profile == '' )?$this->selected:$profile;
		$this->profiles[$profile]['user'] = $user;
	}

	/**
	 * Asigna la contraseña de la base de datos
	 * @param string $password
	 * @param string[optional] $profile
	 * @return void
	 */
	private function setPassword( $password, $profile = '' )
	{
		$profile = ($profile == '' )?$this->selected:$profile;
		$this->profiles[$profile]['password'] = $password;
	}

	/**
	 * Asigna el nombre de la base de datos
	 * @param string $database
	 * @param string[optional] $profile
	 * @return void
	 */
	private function setDatabase( $database, $profile = '' )
	{
		$profile = ($profile == '' )?$this->selected:$profile;
		$this->profiles[$profile]['database'] = $database;
	}

	/**
	 * Crea un nuevo perfil de conexión
	 * @param string $name
	 * @param string $host
	 * @param string $user
	 * @param string $password
	 * @param string $database
	 * @return void
	 */
	public function createProfile( $name, $host, $user, $password, $database )
	{
		$this->setHost( $host, $name );
		$this->setUser( $user, $name );
		$this->setPassword( $password, $name );
		$this->setDatabase( $database, $name );
	}
	
	/**
	 * Selecciona el perfil de conexión
	 * @param string[optional] $name
	 * @return boolean
	 */
	public function selectProfile( $name = 'default' )
	{
		if( isset( $this->profiles[$name] ) )
		{
			$previous = $this->selected;
			$this->selected = $name;
			$this->connected = false;
			if( $this->checkConnection() )
			{
				return true;
			}
			else
			{
				$this->selected = $previous;
				return false;
			}
			
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Ejecuta una sentencia SQL
	 * @param string $query
	 * @return resource
	 */
	public function query( $query )
	{
		if ( !$this->checkConnection() )
		{
			return NULL;
		}
		
		$result = mysql_query($query, $this->connection);
		if( !$result )
		{
			$this->query_history .= 'Error: '.$query."\n";
			trigger_error( 'Ha ocurrido un error al procesar los datos: '.$this->errorMessage()."\n", E_USER_WARNING );
			return NULL;	
		}
		else
		{
			$this->query_history .= $query."\n";
			return $result;
		}
	}
	
	/**
	 * Devuelve los datos de una fila dentro del resultado
	 * @param resource $result
	 * @param int[optional] $type (F_OBJECT, F_ASSOC, F_ROW)
	 * @return mixed
	 */
	public function fetch( $result, $type = self::F_OBJECT )
	{
		switch( $type )
		{
			case self::F_OBJECT:
				return mysql_fetch_object( $result );
			case self::F_ROW:
				return mysql_fetch_row( $result );	
			case self::F_ASSOC:
				return mysql_fetch_assoc( $result );
			default:
				trigger_error( 'Se ha enviado un parametro invalido a la función', E_USER_WARNING );
				return NULL;
		}
	}

	/**
	 * Obtiene un campo específico de una fila
	 * @param resource $result
	 * @param mixed $field
	 * @param int[optional] $row
	 * @return string
	 */
	public function result( $result, $field, $row = 0 )
	{
		return mysql_result( $result, $row, $field );
	}

	/**
	 * Devuelve el id autoincrementado de la ultima fila insertada
	 * @return int
	 */
	public function insertId()
	{
		return mysql_insert_id();
	}
	
	/**
	 * Devuelve el número de filas para el recurso
	 * @param resource $result
	 * @return int
	 */
	public function numRows( $result )
	{
		return mysql_num_rows( $result );
	}
	
	/**
	 * Evalúa los valores a ingresar en las sentencias SQL
	 * @param string $value
	 * @return string
	 */
	public function evalSQL( $value ) 
	{		
		if ( get_magic_quotes_gpc() )
		{
			stripslashes( $value );
		}
		
		if( $this -> checkConnection() )
		{
			$value = mysql_real_escape_string( $value, $this->connection );
		}
		
		return $value;
	}
	
	/**
	 * Devuelve el historial de consultas realizadas
	 * @return string
	 */
	public function showQueries()
	{
		return $this->query_history;
	}

	/**
	 * Verifica si la conexion esta activa
	 * @return boolean
	 */
	public function checkConnection()
	{
		if (!$this->connected)
		{
			return $this->defaultConnection();
		}
		elseif ( !mysql_ping( $this->connection ) )
		{
			mysql_close( $this->connection );
			$this->connected = false;
			return $this->defaultConnection();
		}
		else
		{
			return true;
		}
	}
	
	/**
	 * Conecta con la base de datos
	 * @return boolean
	 */
	private function defaultConnection()
	{
		$connection = @mysql_connect( $this->profiles[$this->selected]['host'], $this->profiles[$this->selected]['user'], $this->profiles[$this->selected]['password'] );
		if (!$connection)
		{
			trigger_error( 'No se pudo establecer una conexión con el perfil '.$this->selected.' '.$this->errorMessage(), E_USER_NOTICE );
			return false;
		}

		$base = @mysql_select_db( $this->profiles[$this->selected]['database'], $connection);
		if(!$base)
		{
			trigger_error( 'No se pudo seleccionar la base de datos '. $this -> errorMessage(), E_USER_WARNING );
			return false;
		}
		
		$this->connected = true;
		$this->connection = $connection;
		
		return true;
	}

	/**
	 * Muestra el mensaje de error de mysql
	 * @return string
	 */
	private function errorMessage()
	{
		return mysql_error();
	}	
	
	/**
	 * Devuelve la instancia única de la clase
	 * @return Database
	 * @static
	 */
	public static function getInstance()
	{
		if( !self::$instance instanceof self )
		{
			self::$instance = new self;
		}
		else
		{
			self::$instance->selectProfile();
		}

		return self::$instance;
	}
	
	/**
	 * Lee los valores de la conexión desde un archivo XML
	 * @param string $file
	 * @return boolean
	 * @static
	 */
	public static function connectFromXML( $file )
	{
		if( !file_exists( $file ) )
		{
			trigger_error( 'No existe archivo de conexión', E_USER_NOTICE ); // @todo localizar contenido
			return false;
		}
		
		$xml = simplexml_load_file( $file );
		$database = self::getInstance();
		
		foreach( $xml->profile as $profile )
		{
			$database->createProfile( (string) $profile['id'], (string) $profile->host, (string) $profile->user, (string) $profile->password, (string) $profile->database );
		}
		
		if( $database->checkConnection() )
		{
			return true;
		}
		else
		{
			return false;
		}
	}
}

// Fin de archivo
