<?php

/**
 * Definición de la clase ErrorHandler
 * @package Core
 * @version 3.0
 * @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License
 * @copyright TuApp.net 2011
 */

/**
 * Gestor de errores personalizado del sistema
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright TuApp.net 2011
 * @package Core
 * @version 3.0
 */
 
class ErrorHandler
{
	/**
	 * Instancia única de la clase
	 * @var ErrorHandler
	 * @static
	 */
	private static $instance;
	
	/**
	 * Controla si se muestran o no los errores
	 * @var boolean
	 */
	private $report = true;
	
	/**
	 * Controla la salida de mensajes de depuración
	 * @var boolean
	 */
	private $debug = true;
	
	/**
	 * Controla la salida de mensajes de depuracion internos de php
	 * @var boolean
	 */
	private $system_info = false;
	
	/**
	 * Controla la salida de mensajes de advertencia sobre fuciones deprecadas
	 * @var boolean
	 */
	private $deprecated = true;
	
	/**
	 * Nombre de la función utilizada para detener la ejecución del sistema
	 * @var callback
	 */
	private $halt_callback = '';
	
	/**
	 * Controla si se ha registrado un error
	 * @var boolean
	 */
	public static $has_errors = false;

	/**
	 * Constructor privado de la clase
	 * Registra el nuevo manejador de errores
	 * @return void
	 */
	private function __construct()
	{
		set_error_handler( array( $this, 'handler' ) );
	}
	
	/**
     * Invalida la clonación de la clase
     * @return void
     * @final
     */
    
   	final public function __clone()
   	{
     	trigger_error("Operación Invalida: No puedes clonar una instancia de ". get_class($this) .".", E_USER_ERROR );
   	}
  
  	/**
  	 * Invalida la deserialización de la clase
  	 * @return void
  	 * @final
  	 */
  	
   	final public function __wakeup()
   	{
      	trigger_error("No puedes deserializar una instancia de ". get_class($this) .".", E_USER_ERROR );
   	}

   	/**
   	 * Modifica los parámetros de activación de los mensajes de depuración del sistema
   	 * @param boolean[optional] $report
   	 * @param boolean[optional] $debug
   	 * @param boolean[optional] $system_info
   	 * @param boolean[optional] $deprecated
   	 * @return void
   	 */
	public function setDebugMode( $report = true, $debug = true, $system_info = false, $deprecated = true )
	{
		$this->report = $report;
		$this->debug = $debug;
		$this->system_info = $system_info;
		$this->deprecated = $deprecated;
	}
	
	/**
	 * Modifica la función utilizada para detener la ejecución del sistema
	 * @param callback $function_name funcion( $error_string, $error_file, $error_line ) 
	 * @return void
	 */	
	public function setHaltHandler( $function_name )
	{
		$this->halt_callback = $function_name;
	}

	/**
	 * Manejador de error personalizado
	 * @param int $error_number
	 * @param string $error_string
	 * @param string $error_file
	 * @param int $error_line
	 * @param array $error_context
	 * @return boolean
	 */
	public function handler( $error_number, $error_string, $error_file, $error_line, $error_context )
	{
		//Si se ha desactivado la generación de errores se sale de la función
		if( error_reporting() == 0 )
		{
			return true;
		}
		
		switch( $error_number )
		{
			case E_USER_ERROR:
				
				$error_log = date( 'd/m/Y H:i:s' )."\n";// @todo localizar contenido
				$error_log .= 'Error Fatal: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n";// @todo localizar contenido
				$error_log .= "Contexto:\n".print_r( $error_context, true )."\n\n";// @todo localizar contenido
				
				file_put_contents( LOGS.'/error.log', $error_log, FILE_APPEND | LOCK_EX );
				
				$halt_callback = $this->halt_callback;
				if( function_exists( $halt_callback ) )
				{
					$halt_callback( $error_string, $error_file, $error_line );
				}
				else
				{
					die( 'Error Fatal: "'.$error_string.'" en '.$error_file.' línea '.$error_line.'. Consulte el log de errores para obtener más detalles.' ); //@todo localizar contenido
				}
				break;
			case E_WARNING:
			case E_USER_WARNING:
				if( !$this->report )
				{
					return true;
				}
				
				self::$has_errors = true;
				
				$buffer = Buffer::getInstance();
				
				$buffer->write( "e_warning", 'Advertencia: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n" );// @todo localizar contenido
				
				break;
			case E_USER_NOTICE:
				if( !$this->debug || !$this->report)
				{
					return true;
				}
				
				self::$has_errors = true;
				
				$buffer = Buffer::getInstance();
				
				$buffer->write( "e_notice", 'Mensaje: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n" );// @todo localizar contenido
								
				break;
			case E_NOTICE:
			case E_STRICT:
				if( !$this->system_info || !$this->report )
				{
					return true;
				}
				
				self::$has_errors = true;
				
				$buffer = Buffer::getInstance();
				
				$buffer->write( "e_notice", 'Mensaje: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n" );// @todo localizar contenido
								
				break;
			case E_DEPRECATED:
			case E_USER_DEPRECATED:
				if( !$this->deprecated || !$this->report )
				{
					return true;
				}
				
				self::$has_errors = true;
				
				$buffer = Buffer::getInstance();
				
				$buffer->write( "e_notice", 'Mensaje: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n" );// @todo Localizar contenido
								
				break;
			default:
				//error desconocido, se pasa el control al gestor por defecto de php
				return false;
		}
				
		return true;
	}

	/**
	 * Devuelve la instancia única de la clase
	 * @return ErrorHandler
	 * @static
	 */
	public static function getInstance()
	{
		if( !self::$instance instanceof self )
		{
			self::$instance = new self;
		}

		return self::$instance;
	}
	
	/**
	 * Inicializa el gestor de errores
	 * @param boolean[optional] $report
	 * @param boolean[optional] $debug
	 * @param boolean[optional] $system_info
	 * @param boolean[optional] $deprecated
	 * @param callback[optional] $halt_callback function( $error_string, $error_file, $error_line )
	 * @return void
	 * @static
	 */
	public static function init( $report = true, $debug = true, $system_info = false, $deprecated = true, $halt_callback = '' )
	{
		$md_error = ErrorHandler::getInstance();
        $md_error->setDebugMode($report, $debug, $system_info, $deprecated);
		$md_error->setHaltHandler( $halt_callback );
	}
	
	/**
	 * Obtiene los mensajes de advertencia del sistema
	 * @return string
	 * @static
	 */
	public static function getWarnings()
	{
		$buffer = Buffer::getInstance();
		return $buffer->getContent('e_warning', false);
	}
	
	/**
	 * Obtiene los mensajes de notificación del sistema
	 * @return string
	 * @static
	 */
	public static function getNotices()
	{
		$buffer = Buffer::getInstance();
		return $buffer->getContent('e_notice', false);
	}
}
 
// Fin de archivo
