<?php

/**
 * Definición de la clase Mailer
 * @package Mailer
 * @version 1.1
 * @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License
 * @copyright TuApp.net 2011
 */

/**
 * Proceso de envío de correo electrónico
 * Basado en PHPMailer
 *
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright TuApp.net 2011
 * @package Mailer
 * @version 1.1
 * @see PHPMailer
 */

class Mailer
{
	/**
	 * Los correos electrónicos se envían mediante la función mail()
	 */
	const MAIL = 10;

	/**
	 * Los correos electrónicos se envían mediante SMTP
	 */
	const SMTP = 20;

	/**
	 * Los correos electrónicos se envían mediante el SMTP de Gmail
	 */
	const GMAIL = 30;

	/**
	 * Almacena las variables de configuración de la clase
	 * @var array
	 * @static
	 */
	
	private static $conf = array();
	
	/**
	 * Devuelve los valores de configuración de la clase
	 * @param string $name
	 * @return string
	 * @static
	 */
	private static function get( $name )
	{
		return self::$conf[$name];
	}
	
	/**
	 * Agrega las direcciones de los remitentes
	 * @param mixed $address puede ser un string o un array
	 * @param PHPMailer $mail
	 * @return void
	 * @static
	 */
	private static function processAddress( $address, PHPMailer $mail )
	{
		if( is_array($address) )
		{
			foreach( $address as $name => $email )
			{
				$mail->AddAddress( $email, $name );
			}
		}
		elseif( $address == '' )
		{
			if( self::get('default_address_email') != '' )
			{
				$mail->AddAddress( self::get('default_address_email'), self::get('default_address_name') );
			}
			else
			{
				trigger_error( 'No hay direcciones de envío', E_USER_WARNING ); //@todo localizar contenido
			}
		}
		else
		{
			$mail->AddAddress( $address );
		}
	}

	/**
	 * Configura el método mediante el cual se enviara el correo electrónico
	 * @param PHPMailer $mail
	 * @return boolan
	 * @static
	 */
	private static function confMailServer( PHPMailer $mail )
	{
		$type = self::get('mail_type');

		switch( $type )
		{
			case self::GMAIL :
				return self::confGmail($mail);
			case self::SMTP :
				return self::confSMTP($mail);
			case self::MAIL :
			default:
				return true;
		}
	}

	/**
	 * Configura el sistema para enviar los correos a través de Gmail
	 * @param PHPMailer $mail
	 * @return boolean
	 * @static
	 */
	private static function confGmail( PHPMailer $mail )
	{
		$gmail_username = self::get('gmail_username');
		$gmail_password = self::get('gmail_password');

		if( $gmail_username == '' || $gmail_password == '' )
		{
			trigger_error( 'No se han configurado todos los parámetros para Gmail (gmail_username, gmail_password)', E_USER_WARNING ); //@todo localizar contenido
			return false;
		}

		$mail->IsSMTP();
		$mail->SMTPAuth   = true;
		$mail->SMTPSecure = "ssl";
		$mail->Host       = "smtp.gmail.com";
		$mail->Port       = 465;
		$mail->Username   = $gmail_username;
		$mail->Password   = $gmail_password;

		return true;
	}

	/**
	 * Configura el sistema para enviar los correos mediante un servidor SMTP
	 * @param PHPMailer $mail
	 * @return boolean
	 * @static
	 */
	private static function confSMTP( PHPMailer $mail )
	{
		$domain = self::get( 'smtp_domain' );
		$username = self::get('smtp_username');
		$password = self::get('smtp_password');
		
		if( $domain == '' || $username == '' || $password == '' )
		{
			trigger_error('No se han configurado todos los parametros para SMTP (smtp_domain, smtp_username, smtp_password)', E_USER_WARNING ); //@todo localizar contenido
			return false;
		}
		
		$mail->IsSMTP();
		$mail->SMTPAuth   = true;
		
		$mail->Host       = $domain;
		$mail->Port       = (self::get('smtp_port') == '')?26:self::get('smtp_port');                    // set the SMTP port for the GMAIL server
		$mail->Username   = $username;
		$mail->Password   = $password;
	}
	
	/**
	 * Configura las direcciones desde las que se envía el correo electrónico
	 * @param PHPMailer $mail
	 * @return void
	 * @static
	 */
	private static function confSender( PHPMailer $mail )
	{
		if( self::get( 'from_email' ) == '' )
		{
			trigger_error( 'No se ha definido la variable from_email', E_USER_NOTICE );
		}
		else
		{
			$mail->SetFrom( self::get( 'from_email' ), self::get( 'from_name' ) );
		}
		
		if( self::get( 'replyto_email') != '' )
		{
			$mail->AddReplyTo( self::get( 'replyto_email' ),  self::get( 'replyto_name' ) );
		}
	}

	/**
	 * Configura los parámetros del sistema
	 * @param string $name
	 * @param string $value
	 * @return void
	 * @static
	 */
	
	public static function set( $name, $value )
	{
		self::$conf[$name] = $value;
	}
	
	/**
	 * Envía el correo electrónico con los parámetros configurados en el sistema
	 * @param string $subject
	 * @param string $body
	 * @param mixed[optional] $address puede ser un array o una string
	 * @return boolean
	 * @static
	 */
	
	public static function sendMail( $subject, $body, $address = '' )
	{
		$mail = new PHPMailer();

		$mail->Subject = $subject;
		$mail->MsgHTML($body);
		
		if( !self::confMailServer($mail) )
		{
			return false;
		}
		
		self::confSender($mail);
		self::processAddress($address, $mail);

		return $mail->Send();
	}

}

// Fin de archivo
